#!/usr/bin/bash

#
#                ╔═══╗╔╗──╔╗╔═══╗╔═╗─╔╗
#                ║╔═╗║║╚╗╔╝║║╔═╗║║║╚╗║║
#                ║║─║║╚╗║║╔╝║╚═╝║║╔╗╚╝║
#                ║║─║║─║╚╝║─║╔══╝║║╚╗║║
#                ║╚═╝║─╚╗╔╝─║║───║║─║║║
#                ╚═══╝──╚╝──╚╝───╚╝─╚═╝
#
# ╔══╗─────────────────╔╗─╔╗     ─────────╔╗──╔╗────────────
# ║╔╗║─────────────────║║─║║     ────────╔╝╚╗╔╝╚╗───────────
# ║╚╝║╔═╗╔╗╔╗╔╗╔══╗╔══╗║╚═╝║     ╔╗╔╗╔══╗╚╗╔╝╚╗╔╝╔══╗╔═╗╔══╗
# ║╔═╝║╔╝─╣║╚╝║║╔╗║║╔═╝╚═╗╔╝     ║╚╝║║╔╗║─║║──║║─║║═╣║╔╝║══╣
# ║║──║║─║║╚╗╔╝║╔╗║║╚═╗╔═╝║─     ║║║║║╔╗║─║╚╗─║╚╗║║═╣║║──══║
# ╚╝──╚╝─╚╝─╚╝─╚╝╚╝╚══╝╚══╝─     ╚╩╩╝╚╝╚╝─╚═╝─╚═╝╚══╝╚╝─╚══╝
#
#   Maintainer: Roman Kutashenko <roman.kutashenko@ovpn.com>
#

# -----------------------------------------------------------------------------

readonly SCRIPT_FOLDER="$(cd "$(dirname "$0")" && pwd)"
readonly COMMAND_MODE=${1}
readonly CSV_DNS_LIST=${2}

readonly RESTORE_FILE="/var/log/OVPN/restore-dns.dat"
readonly BASE_DNS_FILE="/etc/resolv.conf"
readonly LOG_FILE="/var/log/OVPN/dns.log"

# -----------------------------------------------------------------------------
log() {
    echo $@ >> ${LOG_FILE}
}

# -----------------------------------------------------------------------------
dns_lock() {
    chattr +i ${BASE_DNS_FILE} || true
}

# -----------------------------------------------------------------------------
dns_unlock() {
    chattr -i ${BASE_DNS_FILE} || true
}

# -----------------------------------------------------------------------------
save_dns_state() {
	cp -f ${BASE_DNS_FILE} ${RESTORE_FILE}
}

# -----------------------------------------------------------------------------
set_dns() {
    dns_unlock

    echo "" > ${BASE_DNS_FILE}
    for i in $(echo ${1} | tr ',' '\n'); do 
        echo "nameserver ${i}" >> ${BASE_DNS_FILE}
    done
    
    dns_lock
}

# -----------------------------------------------------------------------------
get_dns_state() {
    #   Collect data
    local CVS_DATA=()

    for DNS in $(cat /etc/resolv.conf | grep -v '#' | grep nameserver | awk -F ' ' '{print $2}' | tr -d ' '); do
        if printf '%s\0' "${CVS_DATA[@]}" | grep -Fxq -- "${DNS}"; then
            echo ""
        else
            CVS_DATA+=( "${DNS}" )
        fi
    done
    
    # Print as CVS
    printf -v TMP_JOINED '%s,' "${CVS_DATA[@]}"
    echo "${TMP_JOINED%,}"

    # Stop systemd-resoleved
    systemctl disable systemd-resolved.service >/dev/null 2>&1
    systemctl stop systemd-resolved >/dev/null 2>&1
}

# -----------------------------------------------------------------------------
restore_dns() {
    dns_unlock

	cp -f ${RESTORE_FILE} ${BASE_DNS_FILE}

    # Start systemd-resoleved
    systemctl enable systemd-resolved.service >/dev/null 2>&1
    systemctl start systemd-resolved >/dev/null 2>&1
}

# -----------------------------------------------------------------------------
restart_dns_resolving() {
    echo "" > //dev/null
}

# -----------------------------------------------------------------------------

#
#   Check permissions
#
[ $EUID -ne 0 ] && error_exit "$0 should be run as root" 2

#
#   Process command
#

log "---------- $(date) ----------"
log "COMMAND  : ${COMMAND_MODE}"

#
#	Save DNS servers
#
if [ "${COMMAND_MODE}" == "save" ]; then
    log "Save DNS state"
	save_dns_state
    exit $?
fi

#
#	Restore DNS Servers
#
if [ "${COMMAND_MODE}" == "restore" ]; then
	ls -lah "${RESTORE_FILE}" >> ${LOG_FILE}
    if [ ! -f "${RESTORE_FILE}" ]; then
        log "ERROR: there is no ${RESTORE_FILE}"
        exit -1
    fi
    restore_dns
    rm -rf "${RESTORE_FILE}"
    restart_dns_resolving
    exit $?
fi

#
#	Set DNS Servers
#
if [ "${COMMAND_MODE}" == "set" ]; then
    log "Required DNS list : ${CSV_DNS_LIST}"
	set_dns "${CSV_DNS_LIST}"
    restart_dns_resolving
    exit $?
fi

#
#	Get DNS Servers
#
if [ "${COMMAND_MODE}" == "get" ]; then
    # log "Get current DNS list"
    get_dns_state
    exit $?
fi

log "ERROR: Command is not supported"
exit -1

# -----------------------------------------------------------------------------
